#include <EEPROM.h>
#include <Makeblock.h>
#include <Servo.h>
#include <SoftwareSerial.h>
#include <Wire.h>

int dirPinL = mePort[PORT_1].s1;//the direction pin connect to Base Board PORT1 SLOT1
int stpPinL = mePort[PORT_1].s2;//the Step pin connect to Base Board PORT1 SLOT2
int dirPinR = mePort[PORT_2].s1;//the direction pin connect to Base Board PORT2 SLOT1
int stpPinR = mePort[PORT_2].s2;//the Step pin connect to Base Board PORT2 SLOT2

MePort servoPort(PORT_7);
int servopin =  servoPort.pin2();
Servo servoPen;

/************** motor movements ******************/
void stepperMoveL(int dir)
{
  if(dir>0){
    digitalWrite(dirPinL, HIGH);
  }else{
    digitalWrite(dirPinL, LOW);
  }
  digitalWrite(stpPinL, HIGH);
  delayMicroseconds(10);
  digitalWrite(stpPinL, LOW);
}

void stepperMoveR(int dir)
{
  if(dir>0){
    digitalWrite(dirPinR, LOW);
  }else{
    digitalWrite(dirPinR, HIGH);
  }
  digitalWrite(stpPinR, HIGH);
  delayMicroseconds(10);
  digitalWrite(stpPinR, LOW);
}


/************** calculate movements ******************/
#define STEPDELAY_MIN 200 // micro second
#define STEPDELAY_MAX 1000

#define STEPS_PER_CIRCLE 3200 // Stepper steps/revolution * suvb steps (200*16)
#define WHEEL_DIAMETER 64// wheel diameter mm
#define AXIS_DIAMKETERWIDTH 132 // distance between wheels
#define STEPS_PER_MM 15916L  // (STEPS_PER_CIRCLE / PI / WHEEL_DIAMETER)
#define STEPS_PER_DEGREE 18333L // (STEPS_PER_MM * PI * AXIS_DIAMETER / 360)

/************ turtle Logo commands ******************/

void doMove(long steps, int dirL,int dirR)
{
  for(long i=0; i<steps; i++)
  {
     stepperMoveL(dirL);
     stepperMoveR(dirR);
     delayMicroseconds(STEPDELAY_MAX);
  }
}

void FW(int distance)
{
  long steps;
  steps = distance*STEPS_PER_MM/1000;

  Serial.print("Mode Forward: ");
  Serial.print(distance, DEC);
  Serial.print("mm, ");
  Serial.println(steps, DEC);
  doMove(steps, 1, 1);
}

void BK(int distance)
{
  long steps;
  steps = distance*STEPS_PER_MM/1000;

  Serial.print("Mode Backward: ");
  Serial.print(distance, DEC);
  Serial.print("mm, ");
  Serial.println(steps, DEC);
  doMove(steps, 0, 0);
}

void TR(int degree)
{
  long steps;
  steps = degree*STEPS_PER_DEGREE/1000; 

  Serial.print("Mode Turn Right: ");
  Serial.print(degree, DEC);
  Serial.print(" grad, ");
  Serial.println(steps, DEC);
  doMove(steps, 0, 1);
}

void TL(int degree)
{
  long steps;
  steps = degree*STEPS_PER_DEGREE/1000; 

  Serial.print("Mode Turn Left: ");
  Serial.print(degree, DEC);
  Serial.print(" grad, ");
  Serial.println(steps, DEC);
  doMove(steps, 1, 0);
} 

#define PENUP_POS 80
#define PENDOWN_POS 60

void PU()
{
    servoPen.write(PENUP_POS);
    delay(100);
}

void PD()
{
    servoPen.write(PENDOWN_POS);
    delay(100);
}

void Square(int len, int turn) 
{
  PD();
  for(int i=0; i<4; i++)
  {
    FW(len);
    TR(turn);
  }
  PU();
}

void Nikolaus(int len)
{
  int diagonale = int((float)len*1.41F);
  int roof = int((float)len*0.707F);

  PD();
  FW(len);
  TR(45);
  FW(roof);
  TR(90);
  FW(roof);
  TR(45);
  FW(len);
  TR(135);
  FW(diagonale);
  TR(135);
  FW(len);
  TR(135);
  FW(diagonale);
  TL(135);
  FW(len);
  PU();
} 

void Spiral(int len, int turn, int duration)
{
  PD();
  int act_len = len;
  for(int d=0; d<duration; d++)
  {
    for(int i=0; i<2; i++)
    {
      FW(act_len);
      TR(turn);
    }
    act_len += len;
  }
  PU();

}

void initStepper()
{
  pinMode(dirPinL, OUTPUT);
  pinMode(stpPinL, OUTPUT);
  pinMode(dirPinR, OUTPUT);
  pinMode(stpPinR, OUTPUT);
}


void echoRobotSetup()
{
  Serial.println("Logo Turtle v0.2");
  Serial.println("Valid commands are:");
  Serial.println("------------------------------------------");
  Serial.println("FW <distance> // Go Forward <distance> mm");
  Serial.println("BK <distance> // Go Backward <distance> mm");
  Serial.println("TR <angle>    // Turn right <angle> degree");
  Serial.println("TL <angle>    // Turn left <angle> degree");
  Serial.println("PU            // Pen up");
  Serial.println("PD            // Pen down");
  Serial.println("------------------------------------------");
  Serial.println("Special commands are:");
  Serial.println("------------------------------------------");
  Serial.println("SQ <length> // draw Square <lenght> mm");
  Serial.println("SP <length> // draw Spiral <lenght> mm");
  Serial.println("NK <length> // draw Nikolaus <lenght> mm");
  
}


void parseCmd(char * cmd)
{
  int value;
  boolean err = false;
  
  if (cmd[0] == 'F' && cmd[1] == 'W')
  {
    value = atoi(cmd+2);
    FW(value);
  }
  else if (cmd[0] == 'B' && cmd[1] == 'K')
  {
    value = atoi(cmd+2);
    BK(value);
  }
  else if (cmd[0] == 'T' && cmd[1] == 'R')
  {
    value = atoi(cmd+2);
    TR(value);
  }
  else if (cmd[0] == 'T' && cmd[1] == 'L')
  {
    value = atoi(cmd+2);
    TL(value);
  }
  else if (cmd[0] == 'P'&& cmd[1] == 'D')
  {
    PD();
  }
  else if (cmd[0] == 'P'&& cmd[1] == 'U')
  {
    PU();
  }
  else if (cmd[0] == 'S'&& cmd[1] == 'Q')
  {
    value = atoi(cmd+2);
    Square(value, 90);
  }
  else if (cmd[0] == 'S'&& cmd[1] == 'P')
  {
    value = atoi(cmd+2);
    Spiral(value, 90, 10);
  }
  else if (cmd[0] == 'N'&& cmd[1] == 'K')
  {
    value = atoi(cmd+2);
    Nikolaus(value);
  }
  else if (cmd[0] == 'S'&& cmd[1] == 'V')
  {
    value = atoi(cmd+2);
    servoPen.write(value);
  }
  else
  {
    echoRobotSetup();
    err = true;
  }
  if (!err) 
  {
      Serial.println("OK");
  }
}

/************** arduino ******************/
void setup() {
  Serial.begin(9600);
  servoPen.attach(servopin);
  initStepper();
  echoRobotSetup();
}

char buf[64];
char bufindex;
char buf2[64];
char bufindex2;

void loop() {
  char c = 0;
  if(Serial.available())
  {
    c = Serial.read();
    buf[bufindex++]=c;
    if(c=='\n'){
      parseCmd(buf);
      memset(buf,0,64);
      bufindex = 0;
    }
  }
}
